#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/*
A pizzafutár által kézbesítendő pizzák címeit egy egyirányú, az elején strázsát tartalmazó láncolt listában tároljuk.
A címek pontokat jelentenek $(x,y)$ koordinátákkal.
A lista rendezetlen.

A futár a pizzákat úgy kézbesíti, hogy a még hátralévő címek közül mindig a hozzá (légvonalban) legközelebbi következzen.
A futár a központból indul, melynek koordinátái $(0,0)$.

a) Definiálj típust egy ponthoz és a láncolt listához. 2 pont

b) Írj függvényt, mely egy pontot és egy listát kap paraméterként, és megkeresi a ponthoz legközelebbi listaelemet.
Mivel ezt később törölni szeretnénk a listából, az őt {\bf megelőző} listaelem címével térjen vissza. 4 pont

c) Írj függvényt, amely megkapja a címek láncolt listáját, majd kiszámítja és visszaadja, hogy mekkora utat tesz meg a futár, mialatt az összes pizzát kikézbesíti.
A függvény a listából törölhet. 6 pont
*/

// a

typedef struct { double x, y; } point_t;
typedef struct elem { point_t p; struct elem *next; } list_elem, *list_ptr;

double distance(point_t p1, point_t p2)
{
    return sqrt((p1.x-p2.x)*(p1.x-p2.x) + (p1.y-p2.y)*(p1.y-p2.y));
}

// b
list_ptr nearest(list_ptr list, point_t point)
{
    list_ptr p_nearest = list;
    double mindist;
    for (list_ptr p = list; p->next != NULL; p = p->next) {
        double dist = distance(p->next->p, point);         
        if (p == list || dist < mindist) {
            p_nearest = p;
            mindist = dist;
        }
    }
    return p_nearest;
}

// c
double path(list_ptr list)
{
    double d = 0.0;
    point_t pos = {0.0, 0.0};                   
    while (list->next != NULL) {                
        list_ptr p_nearest = nearest(list, pos);
        d += distance(p_nearest->next->p, pos);
        pos = p_nearest->next->p;               
        list_ptr q = p_nearest->next;
        p_nearest->next = q->next;
        free(q);
    }
    return d;                                   
}


/*
A \texttt{rakomany.txt} szöveges fájl első egész értéke egy darabszámot ($N$) ad meg.
Ezt $N$ sor követi; minden sor egy csomag tömegéből (doublevalós) és értékéből (egész) áll.

a) Írj függvényt, mely paraméterként a szöveges fájl nevét kapja, és az összes csomagot beolvassa egy dinamikusan foglalt tömbbe, amit visszaad.
A függvény a tömb méretét paraméterlistán adja vissza!
5 pont

b) Írj függvényt, mely paraméterként a dinamikus tömböt kapja, és paraméterlistán visszaadja a legkisebb és legnagyobb ár / tömeg arányú csomagok indexeit. 3 pont

c) Írj főprogramot, mely a két fenti függvény segítségével beolvassa az adatokat, meghatározza a szélsőértékeket, végül kiírja a standard outputra a két csomag tömegeit.
A főprogram ne szivárogtasson memóriát. 3 pont
*/

typedef struct { double mass; int value; } package_t;

package_t *read(char *fname, int *n)                  
{
    FILE *f = fopen(fname, "r");                      
    fscanf(f, "%d", n);
    package_t *packages = malloc(sizeof(package_t) * *n);
    int i = 0;
    while (fscanf(f, "%lf%d", &packages[i].mass, &packages[i].value) == 2)
        i++;
    fclose(f);
    return packages;
}

void extrema(package_t *packages, int n, int *maxidx, int *minidx)
{
    *maxidx = *minidx = 0;
    for (int i = 1; i < n; i++)                                   
    {
        if (packages[i].mass / packages[i].value < packages[*minidx].mass / packages[*minidx].value)
            *minidx = i;
        if (packages[i].mass / packages[i].value > packages[*maxidx].mass / packages[*maxidx].value)
            *maxidx = i;
    }
}

int main()
{
    int n, minidx, maxidx;
    package_t *packages = read("rakomany.txt", &n);                
    extrema(packages, n, &minidx, &maxidx);                        
    printf("%f %f", packages[minidx].mass, packages[maxidx].mass);
    free(packages);                                                
    return 0;
}

/*
Írj függvényt, mely a paraméterként kapott sztring minden szavának kezdőbetűjét kisbetűsről nagyra vagy nagybetűsről kicsire fordítja.
A sztringben csak betűk és szóközök vannak.
A szavakat pontosan egy szóköz választja el egymástól.
Az első szó előtt és az utolsó után nincs szóköz.
Ha a sztring tartalma pl. "A nagy macska ugat", akkor a függvény lefutása után a sztring tartalma "a Nagy Macska Ugat". 9 pont
*/

void f(char *str)                                       
{
    for (int i = 0; str[i] != '\0'; i++)                
        if (i == 0 || str[i-1] == ' ')                  
        {
            if (str[i] >= 'A' && str[i] <= 'Z')         
                str[i] += 'a' - 'A';
            else if (str[i] >= 'a' && str[i] <= 'z')
                str[i] += 'A' - 'a';
        }
}